@@ -43,6 +43,13 @@ enum
 
 #define NCPUSTATES 4
 
+enum
+{
+	MIN_UPDATE_INTERVAL = 1 * 1000,
+	MAX_UPDATE_INTERVAL = 100 * 1000
+};
+
+
 struct _ProcConfig
 {
 	gint		width;
@@ -84,7 +91,7 @@ struct _LoadGraph {
     guint allocated;
 
     GdkColor *colors;
-    gfloat **data, **odata;
+    gfloat **data;
     guint data_size;
     guint *pos;
 
@@ -115,7 +122,7 @@ struct _ProcInfo
 	GtkTreeIter	node;
 	GtkTreePath	*path;
 	ProcInfo	*parent;
-	GList		*children;
+	GSList		*children;
 
 	GdkPixbuf	*pixbuf;
 	gchar		*name;
